@@ -2,6 +2,8 @@
 
 from __future__ import print_function
 
+import pytest
+
 from distutils.version import LooseVersion
 from numpy import nan, random
 import numpy as np
@@ -11,25 +13,28 @@ from pandas import (DataFrame, Series, Timestamp,
                     date_range)
 import pandas as pd
 
-from pandas.util.testing import (assert_series_equal,
-                                 assert_frame_equal,
-                                 assertRaisesRegexp)
+from pandas.util.testing import assert_series_equal, assert_frame_equal
 
 import pandas.util.testing as tm
 from pandas.tests.frame.common import TestData, _check_mixed_float
 
 
+try:
+    import scipy
+    _is_scipy_ge_0190 = scipy.__version__ >= LooseVersion('0.19.0')
+except:
+    _is_scipy_ge_0190 = False
+
+
 def _skip_if_no_pchip():
     try:
         from scipy.interpolate import pchip_interpolate  # noqa
     except ImportError:
-        import nose
-        raise nose.SkipTest('scipy.interpolate.pchip missing')
+        import pytest
+        pytest.skip('scipy.interpolate.pchip missing')
 
 
-class TestDataFrameMissingData(tm.TestCase, TestData):
-
-    _multiprocess_can_split_ = True
+class TestDataFrameMissingData(TestData):
 
     def test_dropEmptyRows(self):
         N = len(self.frame.index)
@@ -66,29 +71,31 @@ class TestDataFrameMissingData(tm.TestCase, TestData):
         smaller_frame = frame.dropna()
         assert_series_equal(frame['foo'], original)
         inp_frame1.dropna(inplace=True)
-        self.assert_numpy_array_equal(smaller_frame['foo'], mat[5:])
-        self.assert_numpy_array_equal(inp_frame1['foo'], mat[5:])
+
+        exp = Series(mat[5:], index=self.frame.index[5:], name='foo')
+        tm.assert_series_equal(smaller_frame['foo'], exp)
+        tm.assert_series_equal(inp_frame1['foo'], exp)
 
         samesize_frame = frame.dropna(subset=['bar'])
         assert_series_equal(frame['foo'], original)
-        self.assertTrue((frame['bar'] == 5).all())
+        assert (frame['bar'] == 5).all()
         inp_frame2.dropna(subset=['bar'], inplace=True)
-        self.assertTrue(samesize_frame.index.equals(self.frame.index))
-        self.assertTrue(inp_frame2.index.equals(self.frame.index))
+        tm.assert_index_equal(samesize_frame.index, self.frame.index)
+        tm.assert_index_equal(inp_frame2.index, self.frame.index)
 
     def test_dropna(self):
         df = DataFrame(np.random.randn(6, 4))
         df[2][:2] = nan
 
         dropped = df.dropna(axis=1)
-        expected = df.ix[:, [0, 1, 3]]
+        expected = df.loc[:, [0, 1, 3]]
         inp = df.copy()
         inp.dropna(axis=1, inplace=True)
         assert_frame_equal(dropped, expected)
         assert_frame_equal(inp, expected)
 
         dropped = df.dropna(axis=0)
-        expected = df.ix[lrange(2, 6)]
+        expected = df.loc[lrange(2, 6)]
         inp = df.copy()
         inp.dropna(axis=0, inplace=True)
         assert_frame_equal(dropped, expected)
@@ -96,14 +103,14 @@ class TestDataFrameMissingData(tm.TestCase, TestData):
 
         # threshold
         dropped = df.dropna(axis=1, thresh=5)
-        expected = df.ix[:, [0, 1, 3]]
+        expected = df.loc[:, [0, 1, 3]]
         inp = df.copy()
         inp.dropna(axis=1, thresh=5, inplace=True)
         assert_frame_equal(dropped, expected)
         assert_frame_equal(inp, expected)
 
         dropped = df.dropna(axis=0, thresh=4)
-        expected = df.ix[lrange(2, 6)]
+        expected = df.loc[lrange(2, 6)]
         inp = df.copy()
         inp.dropna(axis=0, thresh=4, inplace=True)
         assert_frame_equal(dropped, expected)
@@ -128,11 +135,11 @@ class TestDataFrameMissingData(tm.TestCase, TestData):
 
         df[2] = nan
         dropped = df.dropna(axis=1, how='all')
-        expected = df.ix[:, [0, 1, 3]]
+        expected = df.loc[:, [0, 1, 3]]
         assert_frame_equal(dropped, expected)
 
         # bad input
-        self.assertRaises(ValueError, df.dropna, axis=3)
+        pytest.raises(ValueError, df.dropna, axis=3)
 
     def test_drop_and_dropna_caching(self):
         # tst that cacher updates
@@ -151,10 +158,10 @@ class TestDataFrameMissingData(tm.TestCase, TestData):
 
     def test_dropna_corner(self):
         # bad input
-        self.assertRaises(ValueError, self.frame.dropna, how='foo')
-        self.assertRaises(TypeError, self.frame.dropna, how=None)
+        pytest.raises(ValueError, self.frame.dropna, how='foo')
+        pytest.raises(TypeError, self.frame.dropna, how=None)
         # non-existent column - 8303
-        self.assertRaises(KeyError, self.frame.dropna, subset=['A', 'X'])
+        pytest.raises(KeyError, self.frame.dropna, subset=['A', 'X'])
 
     def test_dropna_multiple_axes(self):
         df = DataFrame([[1, np.nan, 2, 3],
@@ -175,28 +182,31 @@ class TestDataFrameMissingData(tm.TestCase, TestData):
         assert_frame_equal(inp, expected)
 
     def test_fillna(self):
-        self.tsframe.ix[:5, 'A'] = nan
-        self.tsframe.ix[-5:, 'A'] = nan
+        tf = self.tsframe
+        tf.loc[tf.index[:5], 'A'] = nan
+        tf.loc[tf.index[-5:], 'A'] = nan
 
         zero_filled = self.tsframe.fillna(0)
-        self.assertTrue((zero_filled.ix[:5, 'A'] == 0).all())
+        assert (zero_filled.loc[zero_filled.index[:5], 'A'] == 0).all()
 
         padded = self.tsframe.fillna(method='pad')
-        self.assertTrue(np.isnan(padded.ix[:5, 'A']).all())
-        self.assertTrue((padded.ix[-5:, 'A'] == padded.ix[-5, 'A']).all())
+        assert np.isnan(padded.loc[padded.index[:5], 'A']).all()
+        assert (padded.loc[padded.index[-5:], 'A'] ==
+                padded.loc[padded.index[-5], 'A']).all()
 
         # mixed type
-        self.mixed_frame.ix[5:20, 'foo'] = nan
-        self.mixed_frame.ix[-10:, 'A'] = nan
+        mf = self.mixed_frame
+        mf.loc[mf.index[5:20], 'foo'] = nan
+        mf.loc[mf.index[-10:], 'A'] = nan
         result = self.mixed_frame.fillna(value=0)
         result = self.mixed_frame.fillna(method='pad')
 
-        self.assertRaises(ValueError, self.tsframe.fillna)
-        self.assertRaises(ValueError, self.tsframe.fillna, 5, method='ffill')
+        pytest.raises(ValueError, self.tsframe.fillna)
+        pytest.raises(ValueError, self.tsframe.fillna, 5, method='ffill')
 
         # mixed numeric (but no float16)
         mf = self.mixed_float.reindex(columns=['A', 'B', 'D'])
-        mf.ix[-10:, 'A'] = nan
+        mf.loc[mf.index[-10:], 'A'] = nan
         result = mf.fillna(value=0)
         _check_mixed_float(result, dtype=dict(C=None))
 
@@ -206,7 +216,7 @@ class TestDataFrameMissingData(tm.TestCase, TestData):
         # empty frame (GH #2778)
         df = DataFrame(columns=['x'])
         for m in ['pad', 'backfill']:
-            df.x.fillna(method=m, inplace=1)
+            df.x.fillna(method=m, inplace=True)
             df.x.fillna(method=m)
 
         # with different dtype (GH3386)
@@ -241,10 +251,39 @@ class TestDataFrameMissingData(tm.TestCase, TestData):
         })
 
         expected = df.copy()
-        expected['Date'] = expected['Date'].fillna(df.ix[0, 'Date2'])
+        expected['Date'] = expected['Date'].fillna(
+            df.loc[df.index[0], 'Date2'])
         result = df.fillna(value={'Date': df['Date2']})
         assert_frame_equal(result, expected)
 
+        # with timezone
+        # GH 15855
+        df = pd.DataFrame({'A': [pd.Timestamp('2012-11-11 00:00:00+01:00'),
+                                 pd.NaT]})
+        exp = pd.DataFrame({'A': [pd.Timestamp('2012-11-11 00:00:00+01:00'),
+                                  pd.Timestamp('2012-11-11 00:00:00+01:00')]})
+        assert_frame_equal(df.fillna(method='pad'), exp)
+
+        df = pd.DataFrame({'A': [pd.NaT,
+                                 pd.Timestamp('2012-11-11 00:00:00+01:00')]})
+        exp = pd.DataFrame({'A': [pd.Timestamp('2012-11-11 00:00:00+01:00'),
+                                  pd.Timestamp('2012-11-11 00:00:00+01:00')]})
+        assert_frame_equal(df.fillna(method='bfill'), exp)
+
+    def test_fillna_downcast(self):
+        # GH 15277
+        # infer int64 from float64
+        df = pd.DataFrame({'a': [1., np.nan]})
+        result = df.fillna(0, downcast='infer')
+        expected = pd.DataFrame({'a': [1, 0]})
+        assert_frame_equal(result, expected)
+
+        # infer int64 from float64 when fillna value is a dict
+        df = pd.DataFrame({'a': [1., np.nan]})
+        result = df.fillna({'a': 0}, downcast='infer')
+        expected = pd.DataFrame({'a': [1, 0]})
+        assert_frame_equal(result, expected)
+
     def test_fillna_dtype_conversion(self):
         # make sure that fillna on an empty frame works
         df = DataFrame(index=["A", "B", "C"], columns=[1, 2, 3, 4, 5])
@@ -284,7 +323,7 @@ class TestDataFrameMissingData(tm.TestCase, TestData):
                                  'C': ['foo', 'bar', '?'],
                                  'D': ['foo2', 'bar2', '?']},
                                 index=date_range('20130110', periods=3))
-        self.assert_frame_equal(result, expected)
+        tm.assert_frame_equal(result, expected)
 
         df = pd.DataFrame({'A': [-1, -2, np.nan],
                            'B': [pd.Timestamp('2013-01-01'),
@@ -299,7 +338,7 @@ class TestDataFrameMissingData(tm.TestCase, TestData):
                                  'C': ['foo', 'bar', '?'],
                                  'D': ['foo2', 'bar2', '?']},
                                 index=pd.date_range('20130110', periods=3))
-        self.assert_frame_equal(result, expected)
+        tm.assert_frame_equal(result, expected)
 
     def test_ffill(self):
         self.tsframe['A'][:5] = nan
@@ -315,6 +354,40 @@ class TestDataFrameMissingData(tm.TestCase, TestData):
         assert_frame_equal(self.tsframe.bfill(),
                            self.tsframe.fillna(method='bfill'))
 
+    def test_frame_pad_backfill_limit(self):
+        index = np.arange(10)
+        df = DataFrame(np.random.randn(10, 4), index=index)
+
+        result = df[:2].reindex(index, method='pad', limit=5)
+
+        expected = df[:2].reindex(index).fillna(method='pad')
+        expected.values[-3:] = np.nan
+        tm.assert_frame_equal(result, expected)
+
+        result = df[-2:].reindex(index, method='backfill', limit=5)
+
+        expected = df[-2:].reindex(index).fillna(method='backfill')
+        expected.values[:3] = np.nan
+        tm.assert_frame_equal(result, expected)
+
+    def test_frame_fillna_limit(self):
+        index = np.arange(10)
+        df = DataFrame(np.random.randn(10, 4), index=index)
+
+        result = df[:2].reindex(index)
+        result = result.fillna(method='pad', limit=5)
+
+        expected = df[:2].reindex(index).fillna(method='pad')
+        expected.values[-3:] = np.nan
+        tm.assert_frame_equal(result, expected)
+
+        result = df[-2:].reindex(index)
+        result = result.fillna(method='backfill', limit=5)
+
+        expected = df[-2:].reindex(index).fillna(method='backfill')
+        expected.values[:3] = np.nan
+        tm.assert_frame_equal(result, expected)
+
     def test_fillna_skip_certain_blocks(self):
         # don't try to fill boolean, int blocks
 
@@ -329,18 +402,18 @@ class TestDataFrameMissingData(tm.TestCase, TestData):
         df[3][-4:] = np.nan
 
         expected = df.fillna(value=0)
-        self.assertIsNot(expected, df)
+        assert expected is not df
 
         df.fillna(value=0, inplace=True)
-        assert_frame_equal(df, expected)
+        tm.assert_frame_equal(df, expected)
 
         df[1][:4] = np.nan
         df[3][-4:] = np.nan
         expected = df.fillna(method='ffill')
-        self.assertIsNot(expected, df)
+        assert expected is not df
 
         df.fillna(method='ffill', inplace=True)
-        assert_frame_equal(df, expected)
+        tm.assert_frame_equal(df, expected)
 
     def test_fillna_dict_series(self):
         df = DataFrame({'a': [nan, 1, 2, nan, nan],
@@ -363,7 +436,8 @@ class TestDataFrameMissingData(tm.TestCase, TestData):
         assert_frame_equal(result, expected)
 
         # disable this for now
-        with assertRaisesRegexp(NotImplementedError, 'column by column'):
+        with tm.assert_raises_regex(NotImplementedError,
+                                    'column by column'):
             df.fillna(df.max(1), axis=1)
 
     def test_fillna_dataframe(self):
@@ -403,31 +477,31 @@ class TestDataFrameMissingData(tm.TestCase, TestData):
         assert_frame_equal(result, expected)
 
     def test_fillna_invalid_method(self):
-        with assertRaisesRegexp(ValueError, 'ffil'):
+        with tm.assert_raises_regex(ValueError, 'ffil'):
             self.frame.fillna(method='ffil')
 
     def test_fillna_invalid_value(self):
         # list
-        self.assertRaises(TypeError, self.frame.fillna, [1, 2])
+        pytest.raises(TypeError, self.frame.fillna, [1, 2])
         # tuple
-        self.assertRaises(TypeError, self.frame.fillna, (1, 2))
+        pytest.raises(TypeError, self.frame.fillna, (1, 2))
         # frame with series
-        self.assertRaises(ValueError, self.frame.iloc[:, 0].fillna,
-                          self.frame)
+        pytest.raises(ValueError, self.frame.iloc[:, 0].fillna, self.frame)
 
     def test_fillna_col_reordering(self):
         cols = ["COL." + str(i) for i in range(5, 0, -1)]
         data = np.random.rand(20, 5)
         df = DataFrame(index=lrange(20), columns=cols, data=data)
         filled = df.fillna(method='ffill')
-        self.assertEqual(df.columns.tolist(), filled.columns.tolist())
+        assert df.columns.tolist() == filled.columns.tolist()
 
     def test_fill_corner(self):
-        self.mixed_frame.ix[5:20, 'foo'] = nan
-        self.mixed_frame.ix[-10:, 'A'] = nan
+        mf = self.mixed_frame
+        mf.loc[mf.index[5:20], 'foo'] = nan
+        mf.loc[mf.index[-10:], 'A'] = nan
 
         filled = self.mixed_frame.fillna(value=0)
-        self.assertTrue((filled.ix[5:20, 'foo'] == 0).all())
+        assert (filled.loc[filled.index[5:20], 'foo'] == 0).all()
         del self.mixed_frame['foo']
 
         empty_float = self.frame.reindex(columns=[])
@@ -445,7 +519,7 @@ class TestDataFrameMissingData(tm.TestCase, TestData):
         assert_frame_equal(res, exp)
 
 
-class TestDataFrameInterpolate(tm.TestCase, TestData):
+class TestDataFrameInterpolate(TestData):
 
     def test_interp_basic(self):
         df = DataFrame({'A': [1, 2, np.nan, 4],
@@ -470,7 +544,7 @@ class TestDataFrameInterpolate(tm.TestCase, TestData):
                         'B': [1, 4, 9, np.nan],
                         'C': [1, 2, 3, 5],
                         'D': list('abcd')})
-        with tm.assertRaises(ValueError):
+        with pytest.raises(ValueError):
             df.interpolate(method='not_a_method')
 
     def test_interp_combo(self):
@@ -490,11 +564,12 @@ class TestDataFrameInterpolate(tm.TestCase, TestData):
     def test_interp_nan_idx(self):
         df = DataFrame({'A': [1, 2, np.nan, 4], 'B': [np.nan, 2, 3, 4]})
         df = df.set_index('A')
-        with tm.assertRaises(NotImplementedError):
+        with pytest.raises(NotImplementedError):
             df.interpolate(method='values')
 
     def test_interp_various(self):
         tm._skip_if_no_scipy()
+
         df = DataFrame({'A': [1, 2, np.nan, 4, 5, np.nan, 7],
                         'C': [1, 2, 3, 5, 8, 13, 21]})
         df = df.set_index('C')
@@ -506,8 +581,15 @@ class TestDataFrameInterpolate(tm.TestCase, TestData):
         assert_frame_equal(result, expected)
 
         result = df.interpolate(method='cubic')
-        expected.A.loc[3] = 2.81621174
-        expected.A.loc[13] = 5.64146581
+        # GH #15662.
+        # new cubic and quadratic interpolation algorithms from scipy 0.19.0.
+        # previously `splmake` was used. See scipy/scipy#6710
+        if _is_scipy_ge_0190:
+            expected.A.loc[3] = 2.81547781
+            expected.A.loc[13] = 5.52964175
+        else:
+            expected.A.loc[3] = 2.81621174
+            expected.A.loc[13] = 5.64146581
         assert_frame_equal(result, expected)
 
         result = df.interpolate(method='nearest')
@@ -516,8 +598,12 @@ class TestDataFrameInterpolate(tm.TestCase, TestData):
         assert_frame_equal(result, expected, check_dtype=False)
 
         result = df.interpolate(method='quadratic')
-        expected.A.loc[3] = 2.82533638
-        expected.A.loc[13] = 6.02817974
+        if _is_scipy_ge_0190:
+            expected.A.loc[3] = 2.82150771
+            expected.A.loc[13] = 6.12648668
+        else:
+            expected.A.loc[3] = 2.82533638
+            expected.A.loc[13] = 6.02817974
         assert_frame_equal(result, expected)
 
         result = df.interpolate(method='slinear')
@@ -530,19 +616,14 @@ class TestDataFrameInterpolate(tm.TestCase, TestData):
         expected.A.loc[13] = 5
         assert_frame_equal(result, expected, check_dtype=False)
 
-        result = df.interpolate(method='quadratic')
-        expected.A.loc[3] = 2.82533638
-        expected.A.loc[13] = 6.02817974
-        assert_frame_equal(result, expected)
-
     def test_interp_alt_scipy(self):
         tm._skip_if_no_scipy()
         df = DataFrame({'A': [1, 2, np.nan, 4, 5, np.nan, 7],
                         'C': [1, 2, 3, 5, 8, 13, 21]})
         result = df.interpolate(method='barycentric')
         expected = df.copy()
-        expected.ix[2, 'A'] = 3
-        expected.ix[5, 'A'] = 6
+        expected.loc[2, 'A'] = 3
+        expected.loc[5, 'A'] = 6
         assert_frame_equal(result, expected)
 
         result = df.interpolate(method='barycentric', downcast='infer')
@@ -556,12 +637,12 @@ class TestDataFrameInterpolate(tm.TestCase, TestData):
         _skip_if_no_pchip()
         import scipy
         result = df.interpolate(method='pchip')
-        expected.ix[2, 'A'] = 3
+        expected.loc[2, 'A'] = 3
 
         if LooseVersion(scipy.__version__) >= '0.17.0':
-            expected.ix[5, 'A'] = 6.0
+            expected.loc[5, 'A'] = 6.0
         else:
-            expected.ix[5, 'A'] = 6.125
+            expected.loc[5, 'A'] = 6.125
 
         assert_frame_equal(result, expected)
 
@@ -611,7 +692,7 @@ class TestDataFrameInterpolate(tm.TestCase, TestData):
                         'C': [np.nan, 2, 5, 7],
                         'D': [np.nan, np.nan, 9, 9],
                         'E': [1, 2, 3, 4]})
-        with tm.assertRaises(TypeError):
+        with pytest.raises(TypeError):
             df.interpolate(axis=1)
 
     def test_interp_inplace(self):
@@ -655,10 +736,3 @@ class TestDataFrameInterpolate(tm.TestCase, TestData):
         # all good
         result = df[['B', 'D']].interpolate(downcast=None)
         assert_frame_equal(result, df[['B', 'D']])
-
-
-if __name__ == '__main__':
-    import nose
-    nose.runmodule(argv=[__file__, '-vvs', '-x', '--pdb', '--pdb-failure'],
-                   # '--with-coverage', '--cover-package=pandas.core']
-                   exit=False)
